@@ -122,7 +122,7 @@ class SMTAccessControl implements iAuthenticate
 	    //if user is not authentified, return a user not authentified json error
 	    else
 	    {
-        	$context->closeSession();
+        	$context->closeSession('access control not authentified');
 	    }
 	    
 	    return $isAllowedToExecute;
@@ -150,7 +150,7 @@ class SMTAccessControl implements iAuthenticate
 		else
 		{
 			//if user is not authentified, return a user not authentified json error
-			$context->closeSession();
+			$context->closeSession('basic control not authentified');
 			throw new SMTAuthenticationException( SMTAuthenticationException::ERROR_BAD_LOGIN);
 		}
 		return $isAllowedToExecute;
