@@ -520,8 +520,8 @@ class SMTEthernetService extends SMTService implements SMTIService
     	        
         	    $firewallMode = $this->getFirewallMode();
         	    $hostName = $this->getHostName();
-        	    $ipv4 = $ethernet->getIpv4Config('ETHO');
-        	    $ipv6 = $ethernet->getIpv6Config('ETHO');
+        	    $ipv4 = $ethernet->getIpv4Config('ETH0');
+        	    $ipv6 = $ethernet->getIpv6Config('ETH0');
         	    $ethernetConfig->setHostName($hostName);
         	    $ethernetConfig->setFirewallMode($firewallMode);
         	    $ethernetConfig->setEnhancedSecurity($enhancedSecurity);
