@@ -15,36 +15,41 @@ namespace app\services\setup;
 
 use app\util\SMTRwDto;
 
-class SMTIpv6DTo extends SMTRwDto
+class SMTIpv6Dto extends SMTRwDto
 {
     /**
      * @var boolean
      */    
-	protected $ipv6Active;
+	protected $ipv6Active = false;
 	/**
 	 * @var boolean
 	 */	
-	protected $isDHCP;
+	protected $isDHCP = false;
 	/**
 	 * @var string
 	 */	
-    protected $ip;
+    protected $ip = '';
     /**
      * @var string
      */    
-	protected $gateway;
+	protected $gateway = '';
 	/**
 	 * @var string
 	 */	
-	protected $site;
+	protected $site = '';
 	/**
 	 * @var string
 	 */	
-	protected $linkip;
+	protected $linkip = '';
 	/**
 	 * @var string
 	 */	
-	protected $dns;
+	protected $dns = '';
+	
+	/**
+	 * @var integer
+	 */
+	protected $subnetPrefixLength = 64;
 	
 	public function setMode( $mode )
 	{
@@ -77,6 +82,14 @@ class SMTIpv6DTo extends SMTRwDto
 		$this->dns = $dns;
 	}	
 	
+	public function setSubnetPrefixLength( $subnetPrefixLength )
+	{
+	    if (strlen($subnetPrefixLength) > 0)
+	    {
+	       $this->subnetPrefixLength = intval($subnetPrefixLength);
+	    }
+	}
+	
 	public function isDHCP()
 	{
 		return $this->isDHCP;
@@ -115,8 +128,12 @@ class SMTIpv6DTo extends SMTRwDto
 	public function getDns()
 	{
 		return $this->dns;
-	}	
+	}
 	
+	public function getSubnetPrefixLength()
+	{
+	    return $this->subnetPrefixLength;
+	}
 	
 	public function stripHtmlTags()
 	{
@@ -125,6 +142,7 @@ class SMTIpv6DTo extends SMTRwDto
 		$this->gateway= strip_tags( $this->gateway);
 		$this->dns= strip_tags( $this->dns);
 		$this->linkip= strip_tags( $this->linkip);
+		$this->subnetPrefixLength= strip_tags( $this->subnetPrefixLength);
 	}
 	
 	/**
