@@ -55,6 +55,14 @@ class SMTSwitchDto extends SMTRwDto implements SMTISerializable
 	 * @var string
 	 */
 	protected $globalSwitchEncodedString = NULL;
+	/**
+	 * @var boolean
+	 */
+	protected $hasLicense = FALSE;
+	/**
+	 * @var number
+	 */
+	protected $totalNumberOfPorts = FALSE;
 
 	public function setIndex( $index )
 	{
@@ -99,6 +107,16 @@ class SMTSwitchDto extends SMTRwDto implements SMTISerializable
 		$this->globalSwitchEncodedString = $globalSwitchString;
 	}
 	
+	public function setLicense( $license )
+	{
+	    $this->hasLicense = $license;
+	}
+	
+	public function setTotalNumberOfPorts( $totalNumberOfPorts )
+	{
+	    $this->totalNumberOfPorts = $totalNumberOfPorts;
+	}
+	
 	public function getGlobalSwitchEncodedString()
 	{
 		return $this->globalSwitchEncodedString;
@@ -123,6 +141,17 @@ class SMTSwitchDto extends SMTRwDto implements SMTISerializable
 	{
 		return $this->index;
 	}
+	
+	public function getLicense()
+	{
+	    return $this->hasLicense;
+	}
+	
+	public function getTotalNumberOfPorts()
+	{
+	    return $this->totalNumberOfPorts;
+	}
+	
     /**
      * Serialize as Json data the user object.<br>
      *
