@@ -35,7 +35,7 @@ class SMTSwitchDecoder
     private $switch_string = NULL;    
 
     /**
-     * OSU count per topologies excluding topologies M,N,O,P,Q,V,X,Y where the number of OSU is given directly in the configuration string
+     * OSU count per topologies excluding topologies M,N,O,P,Q,V,W,X,Y where the number of OSU is given directly in the configuration string
      */
     private $osu_Topologies = array("A"=>0,"B"=>1,"C"=>2,"D"=>1,"E"=>0,"F"=>0,"G"=>0,"H"=>0,"I"=>1,"J"=>1,"K"=>0,"L"=>4,"R"=>1, "T"=>2,"U"=>1);  
     
@@ -72,7 +72,7 @@ class SMTSwitchDecoder
     function getIn()
     {
         $type  = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::TOPOLOGY, 1);
-        if ( $type == 'V')
+        if ( $type == 'V' || $type == 'W')
         {
             return 1;
         }
@@ -99,7 +99,7 @@ class SMTSwitchDecoder
         	{
         		$switchOutPut = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::OUT_POSITION, 4);
         	}
-        	if ( $type == 'V' )
+        	if ( $type == 'V' || $type == 'W')
         	{
         	    $switchOutPut = intval( substr($this->getSwitchEncodingString(), SMTSwitchDecoder::IN_POSITION, 3) ) -$this->getPortDistributorNumber() + intval(substr($this->getSwitchEncodingString(), SMTSwitchDecoder::OUT_POSITION_V, 3) ) * $this->getOSUCount() ; 
         	}
@@ -124,7 +124,7 @@ class SMTSwitchDecoder
         	$type  = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::TOPOLOGY, 1);
         	
         	//for M,N,O,P,Q topologies the count of osus is set with 2 digits in the switch string
-        	if ( $type == 'M' || $type == 'N' || $type == 'O' || $type == 'P' || $type == 'Q' || $type == 'V')
+        	if ( $type == 'M' || $type == 'N' || $type == 'O' || $type == 'P' || $type == 'Q' || $type == 'V' || $type == 'W')
         	{
         		$osuCount = intval( substr($this->getSwitchEncodingString(), SMTSwitchDecoder::OSU_COUNT, 2) );
         	}
@@ -165,7 +165,7 @@ class SMTSwitchDecoder
     	$type  = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::TOPOLOGY, 1);
     	
     	//for P,Q topologies there is a distributor OSU
-    	if ( $type == 'P' || $type == 'Q')
+    	if ( $type == 'P' || $type == 'Q' || $type == 'W')
     	{
     		$osuDistributorPos='OSU31';
     	}
@@ -185,7 +185,7 @@ class SMTSwitchDecoder
     	$type  = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::TOPOLOGY, 1);
     	
     	//for P,Q topologies there is a distributor OSU
-    	if ( $type == 'P' || $type == 'Q')
+    	if ( $type == 'P' || $type == 'Q' || $type == 'W')
     	{
     		$osuDistributor = TRUE;
     	}
@@ -202,7 +202,7 @@ class SMTSwitchDecoder
     {
     	$molex = FALSE;
     	$customization = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::CUSTOMISATION, 1);
-    	if ( $customization == 'A' || $customization == 'B' || $customization == 'D' || $customization == 'F' || $customization == 'H' )
+    	if ( $customization == 'A' || $customization == 'B' || $customization == 'D' || $customization == 'F' || $customization == 'H' || $customization == 'P' )
     	{
     		$molex = TRUE;
     	}
@@ -218,7 +218,7 @@ class SMTSwitchDecoder
     {
     	$jdsu = FALSE;
     	$customization = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::CUSTOMISATION, 1);
-    	if ( $customization == '3' || $customization == '4' ||$customization == '6' || $customization == '7' || $customization == '8' || $customization == '9' || $customization == 'C'  || $customization == 'E' || $customization == 'G' )
+    	if ( $customization == '3' || $customization == '4' ||$customization == '6' || $customization == '7' || $customization == '8' || $customization == '9' || $customization == 'C'  || $customization == 'E' || $customization == 'G' || $customization == 'O')
     	{
     		$jdsu= TRUE;
     	}
@@ -234,7 +234,7 @@ class SMTSwitchDecoder
     {
         $uos = FALSE;
         $customization = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::CUSTOMISATION, 1);
-        if ( $customization == 'I' || $customization == 'J' ||$customization == 'K' || $customization == 'L' || $customization == 'M' || $customization == 'N' )
+        if ( $customization == 'I' || $customization == 'J' ||$customization == 'K' || $customization == 'L' || $customization == 'M' || $customization == 'N' || $customization == 'Q')
         {
             $uos = TRUE;
         }
@@ -269,7 +269,7 @@ class SMTSwitchDecoder
     function hasPortsDistributorAtEnd()
     {
         $type  = substr($this->getSwitchEncodingString(), SMTSwitchDecoder::TOPOLOGY, 1);
-        return ( $type == 'V' );
+        return ( $type == 'V' ) || ($type == 'W');
     }
 }
 ?>
\ No newline at end of file
