@@ -19,7 +19,7 @@ $noUploadRunning = SMTUploadStatus::checkNoUploadRunning();
 
 if ( $noUploadRunning )
 {
-    $uploadStatus = upload_file( SMTOtuBackupRestore::OTU_BACKUP_DIR, 'checkBackup' );    
+    $uploadStatus = upload_file( SMTOtuBackupRestore::getBackupDir(), 'checkBackup' );    
     
     if ( $uploadStatus->getUploadErrorStatus() == SMTUploadStatusDto::NO_ERROR )
     {
