@@ -1,7 +1,7 @@
 #!/bin/bash
 
 function enable_autoip {
-  MAC=$(ifconfig eth0 | grep HWaddr | tr -s [:blank:] | tr [A-Z] [a-z] | cut -d ' ' -f 5)
+  MAC=$(ifconfig eth0 | grep ether | tr -s [:blank:] | tr [A-Z] [a-z] | cut -d ' ' -f 3)
   
   echo "enabling autoip"
   echo "mac address for eth0 equals " $MAC
