@@ -2,13 +2,12 @@
 from .qr_data_factory import IQrDataFactory
 from .cdm_proxy import CdmProxy
 
+from json import dumps
+
 
 class QrCdmFactory(IQrDataFactory):
     def __init__(self, cdm: CdmProxy) -> None:
-        if not isinstance(cdm, CdmProxy):
-            raise TypeError
+        self._cdm: CdmProxy = cdm
 
-        self._cdm = cdm
-
-    def create_json_lite(self):
-        return {}
+    def create_json_lite(self) -> str:
+        return dumps(self._cdm.data)
