@@ -3,7 +3,7 @@
 from json import dumps
 
 
-class VzJsonLite:
+class JsonLite:
     def __init__(self) -> None:
         self.data = dict()
         self.format = None
