@@ -1,9 +1,11 @@
-import bz2, os, sys, base64, traceback, hashlib, subprocess, linecache, time, socket, threading
+import bz2, os, sys, base64, traceback, hashlib, subprocess, linecache, time, socket, threading, struct, gzip
 from lxml import etree
 import urllib.request as urlreq
 from urllib.error import HTTPError, URLError
 import importlib
+import shutil
 import viavi.stdhw.dtbutils as dtb
+from tempfile import mkstemp
 
 def PrintException():
     exc_type, exc_obj, tb = sys.exc_info()
@@ -48,6 +50,8 @@ def run_hook(hook, name):
     return True
 
 def split_url(url):
+    if url is not None and url.endswith(".vui"):
+        return (url, 0)
     u = url.split(":")
     index = int(u[0])
     url = ":".join(u[1:])
@@ -116,7 +120,12 @@ def get_user_agent():
 get_user_agent.user_agent = ""
 
 def get_release_xml(url, proxy):
+    if url is not None:
+        vuimode = url.endswith(".vui")
+    else:
+        vuimode = None
     (release, slot) = split_url(url)
+
     release_file = None
     valid = True
     if "http://" in release:
@@ -145,36 +154,72 @@ def get_release_xml(url, proxy):
 
         release_file = "/var/volatile/release.xml"
         req = urlreq.Request(release)
-        try:
-            with open(release_file, "wb") as local_file:
-                f = urlreq.urlopen(req)
-                local_file.write(f.read())
-        except HTTPError as e:
-            print("HTTP Error:%s %s"%(e.code, release))
-            PrintException()
-            return None, None
-        except URLError as e:
-            print("URL Error: %s %s"%(e.reason, release))
-            PrintException()
-            return None, None
 
-        # Download signature
-        if not os.path.exists("/etc/release/no_check"):
-            req = urlreq.Request(release + ".sig")
+        if vuimode:
             try:
-                with open(release_file + ".sig", "wb") as local_file:
+                with open(release_file, "wb") as local_file:
+                    vui = urlreq.urlopen(req)
+                    header = vui.read(12)
+                    (v,u,i,version,lenxml,lensig) = struct.unpack("<cccBII", header)
+                    xmlc = vui.read(lenxml)
+                    xmlp = gzip.decompress(xmlc)
+                    sigc = vui.read(lensig)
+                    sigp = gzip.decompress(sigc)
+                    local_file.write(xmlp)
+                    if not os.path.exists("/etc/release/no_check"):
+                        with open(release_file + ".sig", "wb") as local_file:
+                            local_file.write(sigp)
+            except HTTPError as e:
+                print("HTTP Error:%s %s"%(e.code, release))
+                PrintException()
+                return None, None
+            except URLError as e:
+                print("URL Error: %s %s"%(e.reason, release))
+                PrintException()
+                return None, None
+        else:
+            try:
+                with open(release_file, "wb") as local_file:
                     f = urlreq.urlopen(req)
                     local_file.write(f.read())
             except HTTPError as e:
                 print("HTTP Error:%s %s"%(e.code, release))
                 PrintException()
-                valid = False
+                return None, None
             except URLError as e:
-                print("URL Error:%s %s"%(e.reason, release))
+                print("URL Error: %s %s"%(e.reason, release))
                 PrintException()
-                valid = False
+                return None, None
+
+            # Download signature
+            if not os.path.exists("/etc/release/no_check"):
+                req = urlreq.Request(release + ".sig")
+                try:
+                    with open(release_file + ".sig", "wb") as local_file:
+                        f = urlreq.urlopen(req)
+                        local_file.write(f.read())
+                except HTTPError as e:
+                    print("HTTP Error:%s %s"%(e.code, release))
+                    PrintException()
+                    valid = False
+                except URLError as e:
+                    print("URL Error:%s %s"%(e.reason, release))
+                    PrintException()
+                    valid = False
     else:  # USB Media
-        release_file = release
+        if vuimode:
+            with open(release, "rb") as vui, open("/var/volatile/release.xml", "wb") as xml, open("/var/volatile/release.xml.sig", "wb") as sig:
+                header = vui.read(12)
+                (v,u,i,version,lenxml,lensig) = struct.unpack("<cccBII", header)
+                xmlc = vui.read(lenxml)
+                xmlp = gzip.decompress(xmlc)
+                xml.write(xmlp)
+                sigc = vui.read(lensig)
+                sigp = gzip.decompress(sigc)
+                sig.write(sigp)
+                release_file = "/var/volatile/release.xml"
+        else:
+            release_file = release
 
     # Check signature
     if not os.path.exists("/etc/release/no_check"):
@@ -267,9 +312,11 @@ class ReleaseXML(object):
         self.__root = etree.parse(release_file)
         self.__url = url
         if url is not None:
+            self.__vui = url.endswith(".vui")
             (self.__release, self.__slot) = split_url(url)
             self.__root.xpath("/root")[0].attrib['url'] = url
         else:
+            self.__vui = False
             url = self.__root.xpath("/root/@url")[0]
             (self.__release, self.__slot) = split_url(url)
 
@@ -289,12 +336,14 @@ class ReleaseXML(object):
         return (etree.tostring(self.__root))
 
     def remote(self):
-        (release, slot) = split_url(self.__url)
-        return ("http://" in release)
+        return ("http://" in self.__release)
 
     def xml_apply_to_product(self):
         return self.__root.xpath("/root/release/@device")[0] == dtb.getMachineName()
 
+    def is_vui(self):
+        return self.__vui
+
 
 def current_release_xml():
     if not os.path.isfile("/etc/release/current.xml"):
@@ -406,4 +455,95 @@ def show_highlighted(url, proxy):
     tmp.append("OK")
     return "\n".join(tmp)
 
-PYTHON3_VIAVI_UPGRADE_VERSION = "1.0.0"
+def remove_old_files(rootpath):
+    # Remove old files form /users/disk declared in /etc/release/remove
+    if os.path.isdir(f"{rootpath}/etc/release/remove"):
+        for f in os.listdir(f"{rootpath}/etc/release/remove"):
+            with open(f"{rootpath}/etc/release/remove/%s" % f) as rf:
+                for line in [l.strip() for l in rf.readlines() if len(l.strip()) > 0 and not l.startswith("#")]:
+                    tab = line.split(" ")
+                    if os.path.isfile(f"{rootpath}/{tab[1]}"):
+                        if md5(tab[1]) == tab[0]:
+                            print("Delete removable file: %s" % tab[1])
+                            os.unlink(f"{rootpath}/{tab[1]}")
+
+def extract_user_from_passwd(filename):
+    users = []
+    if not os.path.isfile(filename):
+        return []
+    with open(filename, mode="r") as fp:
+        for line in fp:
+            line_part = line.split(":")
+            if len(line_part) >= 3:
+                user_id = int(line_part[2])
+                if user_id == 0 or user_id >= 1000:
+                    users.append(line_part[0])
+    return users
+
+def sync_password(src, dest):
+    current_users = extract_user_from_passwd(f"{src}/etc/passwd")
+    new_users = extract_user_from_passwd(f"{dest}/etc/passwd")
+
+    #  2. Keep only password to update
+    user_to_synchronize = [user for user in new_users if user in current_users]
+    current_password = {}
+    with open("/etc/shadow", mode="r") as fp:
+        for line in fp:
+            user = line.split(":")[0]
+            if user in user_to_synchronize:
+                current_password[user] = line.split(":")[1]
+
+    new_shadow, abs_path = mkstemp()
+    with os.fdopen(new_shadow, 'w') as new_file:
+        with open("/mnt/fs/etc/shadow", mode="r") as fp:
+            for line in fp:
+                split_line = line.split(":")
+                user = split_line[0]
+                if user in user_to_synchronize:
+                    new_password = split_line[1]
+                    if user in current_password.keys() and new_password != current_password[user]:
+                        split_line[1] = current_password[user]
+                        new_file.write(":".join(split_line))
+                    else:
+                        new_file.write(line)
+                else:
+                    new_file.write(line)
+    shutil.copymode("/mnt/fs/etc/shadow", abs_path)
+    # Remove original file
+    os.remove("/mnt/fs/etc/shadow")
+    # Move new file
+    shutil.move(abs_path, "/mnt/fs/etc/shadow")
+
+def synchronize(source, destination, overwrite=False):
+    if os.path.isdir(source):
+        if os.path.isdir(destination):
+            for item in os.listdir(source):
+                s = os.path.join(source, item)
+                d = os.path.join(destination, item)
+                synchronize(s, d)
+        else:
+            shutil.copytree(source, destination, symlinks=True)
+    else:
+        # If a file (or link) is marked as to be synchronized and overwrite is enabled,
+        # we want to delete it if it disappeared from the source but still exists in the destination
+        if overwrite and not os.path.lexists(source) and os.path.lexists(destination):
+            os.remove(destination)
+        else:
+            if not os.path.lexists(destination) or overwrite:
+                if os.path.islink(source):
+                    linkto = os.readlink(source)
+                    # We try to create the symlink
+                    # If it already exists AND overwrite is enabled,
+                    # we have to remove the old symlink and create a new one - if they are different
+                    if overwrite and os.path.lexists(destination) and (linkto != os.readlink(destination)):
+                        os.remove(destination)
+                    # If it already exists - do nothing
+                    if not os.path.lexists(destination):
+                        os.symlink(linkto, destination)
+                else:
+                    if not os.path.exists(os.path.dirname(destination)):
+                        os.makedirs(os.path.dirname(destination))
+                    if os.path.exists(source):
+                        shutil.copy2(source, destination)
+
+PYTHON3_VIAVI_UPGRADE_VERSION = "3.1.0"
