@@ -4,30 +4,13 @@ from tempfile import mkstemp
 from os import fdopen, remove
 from shutil import move, copymode
 from viavi.stdhw.bootutils import bootenv
-from viavi.stdhw.blockutils import mount, umount, mount_bind, mounted_point
+from viavi.stdhw.blockutils import mount, umount, mount_bind, mounted_point, sync
 from viavi.upgrade.flashutils import mount_chroot, umount_chroot, chroot_exec, format_device
-from viavi.upgrade import PrintException, run_hook, split_url, md5
+from viavi.upgrade import PrintException, run_hook, split_url, md5, get_user_agent, synchronize, remove_old_files, sync_password
+from urllib.error import HTTPError, URLError
+import struct
 
 
-def synchronize(source, destination, overwrite=False):
-    if os.path.isdir(source):
-        if os.path.isdir(destination):
-            for item in os.listdir(source):
-                s = os.path.join(source, item)
-                d = os.path.join(destination, item)
-                synchronize(s, d)
-        else:
-            shutil.copytree(source, destination, symlinks=True)
-    else:
-        if not os.path.lexists(destination) or overwrite:
-            if os.path.islink(source):
-                linkto = os.readlink(source)
-                os.symlink(linkto, destination)
-            else:
-                if not os.path.exists(os.path.dirname(destination)):
-                    os.makedirs(os.path.dirname(destination))
-                if os.path.exists(source):
-                    shutil.copy2(source, destination)
 
 
 class MyFileObj(io.FileIO):
@@ -41,7 +24,7 @@ class MyFileObj(io.FileIO):
         percent = int(self.tell() * 100 / self.__fsize)
         if percent != self.lastpercent:
             if self.__cls is not None:
-                self.__cls.expose("Upgrade", 50 + (percent * 0.4), self.__rname, "Installation", percent)
+                self.__cls.expose("Upgrade", int(50 + (percent * 0.4)), self.__rname, "Installation", int(percent))
             self.lastpercent = percent
         return io.FileIO.read(self, size)
 
@@ -92,6 +75,12 @@ def download_file(src, dest, proxy, chunk_size=8192, report_hook=None):
 def start_upgrade_tar(xmlrelease, hook=None, options={}, smartmanager=None):
     if not run_hook(hook, 'pre_tar_upgrade'):
         return False
+
+    if xmlrelease.is_vui():
+        return start_upgrade_vui(xmlrelease, hook, options, smartmanager)
+
+    print("Start Upgrade with TAR")
+
     # Option default values
     if 'auto_switch' not in options.keys():
         options['auto_switch'] = True
@@ -148,7 +137,7 @@ def start_upgrade_tar(xmlrelease, hook=None, options={}, smartmanager=None):
         if int(percent) != int(chunk_report.lastpercent):
             if smartmanager is not None:
                 # Dowload during 0%~45% of total
-                smartmanager.expose("Upgrade", percent * 0.45, name, "Download release", percent)
+                smartmanager.expose("Upgrade", int(percent * 0.45), name, "Download release", int(percent))
             chunk_report.lastpercent = percent
 
     chunk_report.lastpercent = 1000
@@ -278,7 +267,14 @@ def start_upgrade_tar(xmlrelease, hook=None, options={}, smartmanager=None):
                 if bk.startswith("@"):
                     override = True
                     bk = bk[1:].strip()
-                for fn in glob.glob(options['current_rootfs'] + bk):
+
+                # Enable deleting files
+                # glob.glob returns nothing but we want to delete the file pointed by bk anyway
+                files = glob.glob(options['current_rootfs'] + bk)
+                if(override and (len(files) == 0)):
+                    files = [ options['current_rootfs'] + bk ]
+
+                for fn in files:
                     if len(options['current_rootfs']) > 0:
                         print("Sync {} {}".format(fn, fn.replace(options['current_rootfs'], "/mnt/fs")))
                         synchronize(fn, fn.replace(options['current_rootfs'], "/mnt/fs"), override)
@@ -295,63 +291,10 @@ def start_upgrade_tar(xmlrelease, hook=None, options={}, smartmanager=None):
         os.system("ln -s /etc/release/%s.xml /mnt/fs/etc/release/current.xml" % id)
 
         # Remove old files form /users/disk declared in /etc/release/remove
-        if os.path.isdir("/mnt/fs/etc/release/remove"):
-            for f in os.listdir("/mnt/fs/etc/release/remove"):
-                with open("/mnt/fs/etc/release/remove/%s" % f) as rf:
-                    for line in [l.strip() for l in rf.readlines() if len(l.strip()) > 0 and not l.startswith("#")]:
-                        tab = line.split(" ")
-                        if os.path.isfile(tab[1]):
-                            if md5(tab[1]) == tab[0]:
-                                print("Delete removable file: %s" % tab[1])
-                                os.unlink(tab[1])
+        remove_old_files("/mnt/fs")
 
         # Manage password changes
-        #  1. Get a list of users before and after the upgrade (only user not system role)
-        def extract_user_from_passwd(filename):
-            users = []
-            if not os.path.isfile(filename):
-                return []
-            with open(filename, mode="r") as fp:
-                for line in fp:
-                    line_part = line.split(":")
-                    if len(line_part) >= 3:
-                        user_id = int(line_part[2])
-                        if user_id == 0 or user_id >= 1000:
-                            users.append(line_part[0])
-            return users
-
-        current_users = extract_user_from_passwd("/etc/passwd")
-        new_users = extract_user_from_passwd("/mnt/fs/etc/passwd")
-
-        #  2. Keep only password to update
-        user_to_synchronize = [user for user in new_users if user in current_users]
-        current_password = {}
-        with open("/etc/shadow", mode="r") as fp:
-            for line in fp:
-                user = line.split(":")[0]
-                if user in user_to_synchronize:
-                    current_password[user] = line.split(":")[1]
-
-        new_shadow, abs_path = mkstemp()
-        with fdopen(new_shadow, 'w') as new_file:
-            with open("/mnt/fs/etc/shadow", mode="r") as fp:
-                for line in fp:
-                    split_line = line.split(":")
-                    user = split_line[0]
-                    if user in user_to_synchronize:
-                        new_password = split_line[1]
-                        if user in current_password.keys() and new_password != current_password[user]:
-                            split_line[1] = current_password[user]
-                            new_file.write(":".join(split_line))
-                        else:
-                            new_file.write(line)
-                    else:
-                        new_file.write(line)
-        copymode("/mnt/fs/etc/shadow", abs_path)
-        # Remove original file
-        remove("/mnt/fs/etc/shadow")
-        # Move new file
-        move(abs_path, "/mnt/fs/etc/shadow")
+        sync_password("", "/mnt/fs")
 
         # Manage Downgrade to fido (cortexa9_vfp_neon architecture)
         if "cortexa9_vfp_neon" in xmlrelease.xpath("/repositories/rpm/@id"):
@@ -425,3 +368,265 @@ def start_upgrade_tar(xmlrelease, hook=None, options={}, smartmanager=None):
     if not run_hook(hook, 'post_tar_upgrade'):
         return False
     return result
+
+
+def start_upgrade_vui(xmlrelease, hook=None, options={}, smartmanager=None):
+    # Option default values
+    print("Start Upgrade with VUI")
+
+    if 'auto_switch' not in options.keys():
+        options['auto_switch'] = True
+    if 'test_update_part' not in options.keys():
+        options['test_update_part'] = False
+    if 'proxy' not in options.keys():
+        options['proxy'] = ''
+    if 'current_rootfs' not in options.keys():
+        options['current_rootfs'] = ''
+    if 'syncronise' not in options.keys():
+        options['syncronise'] = True
+
+    result = True
+    tarf = xmlrelease.xpath("repositories/archive/@file")[0]
+    md5f = xmlrelease.xpath("repositories/archive/@md5")[0]
+    name = xmlrelease.xpath("@name")[0]
+    sizef = int(xmlrelease.xpath("repositories/archive/@size")[0])
+    url = xmlrelease.url()
+    remote = xmlrelease.remote()
+    (xmlpath, slot) = split_url(url)
+    path = "/".join(xmlpath.split("/")[:-1]) + "/" + tarf
+
+    try:
+        try:
+            bootenv.refresh()
+            mmc_part = int(bootenv.mmc_part)
+            if mmc_part == options['parts'][0]:
+                mmc_part_new = options['parts'][1]
+            else:
+                mmc_part_new = options['parts'][0]
+        except:
+            mmc_part_new = options['parts'][1]
+
+        print("mmc_part is %s" % mmc_part_new)
+
+        # Unmount the partition about to be formatted (all mounts)
+        for mount_point in mounted_point(options['flash_block_device'] + "p%s" % mmc_part_new):
+            umount(mount_point)
+
+        run_hook(hook, 'pre_tar_install')
+        if smartmanager is not None:
+            smartmanager.set_action("1: Format partitions")
+        if not os.path.exists("/mnt/fs"):
+            os.makedirs("/mnt/fs")
+        if smartmanager is not None:
+            smartmanager.expose("Upgrade", 1, name, "Prepare filesystem %s" % mmc_part_new, 20)
+        format_device(mmc_part_new)
+        if smartmanager is not None:
+            smartmanager.expose("Upgrade", 2, name, "Prepare filesystem", 40)
+        mount(options['flash_block_device'] + "p%s" % mmc_part_new, "/mnt/fs/", "ext4")
+        if smartmanager is not None:
+            smartmanager.expose("Upgrade", 3, name, "Prepare filesystem", 60)
+        if not os.path.exists("/mnt/fs/user/disk"):
+            os.makedirs("/mnt/fs/user/disk")
+        if smartmanager is not None:
+            smartmanager.expose("Upgrade", 4, name, "Prepare filesystem", 80)
+        mount_bind("/user/disk", "/mnt/fs/user/disk")
+        if smartmanager is not None:
+            smartmanager.expose("Upgrade", 5, name, "Prepare filesystem", 100)
+            smartmanager.set_action("2: Install release")
+
+        vui = None
+        if remote:
+            # Download XML file
+            proxy = options['proxy']
+            if proxy is not None and proxy.strip() != "DIRECT":
+                print("Enable proxy %s" % proxy)
+                proxy_handler = urlreq.ProxyHandler({'http': proxy})
+            else:
+                print("Disable proxy")
+                proxy_handler = urlreq.ProxyHandler({})
+            opener = urlreq.build_opener(proxy_handler)
+            opener.addheaders = [('User-Agent', get_user_agent())]
+            urlreq.install_opener(opener)
+
+            # Check mymirror.php
+            release = xmlrelease.url()
+            base_url = release.replace("http://", "").split("/")[0]
+            mymirror_url = "http://%s/mymirror.php" % (base_url)
+            req = urlreq.Request(mymirror_url)
+            try:
+                mirror = urlreq.urlopen(req).read().decode('ascii')
+                release = release.replace(base_url, mirror)
+                url = url.replace(base_url, mirror)
+                print("Release with mirror: %s" % release)
+            except HTTPError:
+                pass
+
+            req = urlreq.Request(release)
+            vui = urlreq.urlopen(req)
+        else:
+            vui = open("%s" % (release), "rb")
+
+        header = vui.read(12)
+        (v,u,i,version,lenxml,lensig) = struct.unpack("<cccBII", header)
+        xmlc = vui.read(lenxml)
+        sigc = vui.read(lensig)
+
+        def print_gui(self, percent):
+            if smartmanager is not None:
+                smartmanager.expose("Upgrade", 6 + int(percent * 90/100), name, "Installation", percent)
+
+        def StreamDecorator(original, tarsize, notify):
+            setattr(original.__class__, "oldread", original.read)
+            setattr(original.__class__, "notify", notify)
+            original.lastpercent = 0
+            original.tarsize = int(tarsize)
+            original.currentsize = 0
+            original.dlmd5 = hashlib.md5()
+
+            def myread(self, size):
+                percent = int(self.currentsize * 100 / self.tarsize)
+                if percent != self.lastpercent:
+                    self.notify(percent)
+                    self.lastpercent = percent
+
+                if self.currentsize == 0:
+                    key = struct.pack("cccB", b'V', b'U', b'I', 1)
+                    data = self.oldread(4)
+                    chuck = struct.pack("<I", struct.unpack('<I', data)[0] ^ struct.unpack('<I', key)[0])
+                    chuck += self.oldread(size - 4)
+                    self.currentsize += size
+                else:
+                    chuck = self.oldread(size)
+
+                self.dlmd5.update(chuck)
+                self.currentsize += size
+                return chuck
+
+            myread.__doc__ = "My read"
+            myread.__name__ = "read"
+            setattr(original.__class__, "read", myread)
+            return original
+
+        run_hook(hook, 'pre_tar_untar')
+        f = StreamDecorator(vui, sizef, print_gui)
+        tar = tarfile.open(fileobj=f, mode='r|gz', errorlevel=1)
+        tar.extractall("/mnt/fs")
+        sync()
+        run_hook(hook, 'post_tar_untar')
+
+
+        if smartmanager is not None:
+            smartmanager.set_action("5: Check release")
+            smartmanager.expose("Upgrade", 96, name, "Check integrity", 100)
+        if md5f.lower() != f.dlmd5.hexdigest().lower():
+            print("BAD SIGNATURE %s %s", md5f.lower(), f.dlmd5.hexdigest().lower())
+            if options['daemon']:
+                if smartmanager is not None:
+                    smartmanager.upgrade_by_tar_terminate("BAD SIGNATURE")
+            return
+
+        try:
+            shutil.copyfile("/etc/resolv.conf", "/mnt/fs/etc/resolv.conf")
+        except shutil.SameFileError:
+            pass
+        except FileNotFoundError:
+            try:
+                open("/mnt/fs/etc/resolv.conf", 'a').close()
+            except:
+                print("Warning: Can't create /mnt/fs/etc/resolv.conf")
+                pass
+
+        # Copy all file explicitly listed in /etc/release/backup
+        run_hook(hook, 'pre_synchronize')
+        if options['syncronise']:
+            if smartmanager is not None:
+                smartmanager.set_action("6: Synchronize")
+            backup = []
+            if os.path.exists("{}/etc/release/backup".format(options['current_rootfs'])):
+                for f in os.listdir("{}/etc/release/backup".format(options['current_rootfs'])):
+                    with open("{}/etc/release/backup/{}".format(options['current_rootfs'], f)) as fi:
+                        backup += [line.strip() for line in fi if line.strip() != '']
+            if os.path.exists("/mnt/fs/etc/release/backup"):
+                for f in os.listdir("/mnt/fs/etc/release/backup"):
+                    with open("/mnt/fs/etc/release/backup/%s" % f) as fi:
+                        backup += [line.strip() for line in fi if line.strip() != '']
+            run_hook(hook, 'pre_synchronize_list')
+
+            for bk in set(backup):
+                override = False
+                if bk.startswith("@"):
+                    override = True
+                    bk = bk[1:].strip()
+                for fn in glob.glob(options['current_rootfs'] + bk):
+                    if len(options['current_rootfs']) > 0:
+                        print("Sync {} {}".format(fn, fn.replace(options['current_rootfs'], "/mnt/fs")))
+                        synchronize(fn, fn.replace(options['current_rootfs'], "/mnt/fs"), override)
+                    else:
+                        print("Sync {} {}".format(fn, "/mnt/fs/%s" % fn))
+                        synchronize(fn, "/mnt/fs/%s" % fn, override)
+        run_hook(hook, 'post_synchronize')
+
+        if smartmanager is not None:
+            smartmanager.set_action("7: Run post install (don't turn off device)")
+        id = xmlrelease.xpath("@id")[0]
+        xmlrelease.save("/mnt/fs/etc/release/%s.xml" % id)
+        os.system("rm -f /mnt/fs/etc/release/current.xml")
+        os.system("ln -s /etc/release/%s.xml /mnt/fs/etc/release/current.xml" % id)
+
+        # Remove old files form /users/disk declared in /etc/release/remove
+        remove_old_files("/mnt/fs")
+
+        # Manage password changes
+        sync_password("", "/mnt/fs")
+
+        try:
+            mount_chroot()
+            os.system("mount > /tmp/mount.log")
+            chroot_exec("/mnt/fs", ["/usr/sbin/run-postinsts"])
+        except:
+            result = False
+            PrintException()
+        finally:
+            umount_chroot()
+    except subprocess.CalledProcessError as err:
+        result = False
+        PrintException()
+    except:
+        result = False
+        PrintException()
+    finally:
+        if smartmanager is not None:
+            smartmanager.set_action("8: Clean installation")
+            smartmanager.expose("Upgrade", 90, name, "Clean installation", 100)
+        umount("/mnt/fs/user/disk")
+        if os.path.isdir("/mnt/fs/var/lib") and not os.path.isdir("/mnt/fs/var/lib/release-manager"):
+            os.mkdir("/mnt/fs/var/lib/release-manager")
+        with open("/mnt/fs/var/lib/release-manager/upgrade_status", "w") as f:
+            f.write("Success")
+        umount("/mnt/fs")
+        if result:
+            run_hook(hook, 'pre_enable')
+            if smartmanager is not None:
+                smartmanager.set_action("8: Enable new installation")
+                smartmanager.expose("Upgrade", 100, name, "Enable new installation", 100)
+            if options['auto_switch']:
+                print("Setting permanent boot part")
+                bootenv.mmc_part = mmc_part_new
+            if options['test_update_part']:
+                print("Setting temp boot part")
+                bootenv.mmc_part_volatile = mmc_part_new
+            if smartmanager is not None:
+                smartmanager.expose("Upgrade", 100, name, "Finished successful", 100)
+            run_hook(hook, 'post_enable')
+    if options['daemon']:
+        if smartmanager is not None:
+            smartmanager.set_action("Upgrade Terminate")
+        if result:
+            if smartmanager is not None:
+                smartmanager.upgrade_by_tar_terminate("OK")
+        else:
+            if smartmanager is not None:
+                smartmanager.upgrade_by_tar_terminate("FAIL")
+    if not run_hook(hook, 'post_tar_upgrade'):
+        return False
+    return result
