@@ -1,6 +1,6 @@
 from viavi.evm import *
 from viavi.diag import *
-import jdsu.mts as mts
+from viavi.mts import getMachineName
 
 LedsConfigurations = {
     "mts4000ng": {
@@ -48,7 +48,7 @@ LedsConfigurations = {
 
 @DIAG("Led test")
 def t_led(cmd):
-    conf = LedsConfigurations[mts.getMachineName()]
+    conf = LedsConfigurations[getMachineName()]
     try:
         (command,led,mode) = cmd.split()
     except:
