@@ -1,8 +1,8 @@
 from viavi.evm import *
 from viavi.diag import *
 from time import time, sleep
-from jdsu.mts.uts_bdt import UtsBdt
-from jdsu.mts.evdev import InputDevice
+from viavi.mts.uts_bdt import UtsBdt
+from viavi.mts.evdev import InputDevice
 import select
 import os
 import os.path
